/*
  Midnight, a C++ midi library to load and play midi files.
  Copyright (C) 1999 Rodrigo Reyes, reyes@chez.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "AlphaKaraoke.h"
#include <gtk/gtkenums.h>
#include <gtk/gtkrange.h>

char* AlphaKaraoke::DEFAULTFONT =  "-*-courier-*-*-*-*-24-*-*-*-*-*-*-*";

int alphakara_scroll_timeout(gpointer data)
{
  ((AlphaKaraoke*)data)->scrollArea();
  return TRUE;
}

static gint alphakara_expose_event(GtkWidget *widget, GdkEventExpose* event, gpointer data)
{
  //  ((AlphaKaraoke*)data)->computeLines();
  ((AlphaKaraoke*)data)->draw(event);
  return FALSE;
}

static gint alphakara_configure_event(GtkWidget* widget, GdkEventConfigure* event, gpointer data)
{
  //  ((AlphaKaraoke*)data)->updatePixmapBuffer();
  //  ((AlphaKaraoke*)data)->redrawAllLinesPixmaps();
  ((AlphaKaraoke*)data)->update();
  return TRUE;
}

static gint alphakara_delete_event(GtkWidget* widget, GdkEventConfigure* event, gpointer data)
{
  ((AlphaKaraoke*)data)->hide();
  return TRUE;
}

enum {
  TARGET_STRING,
  TARGET_ROOTWIN
};

static GtkTargetEntry target_table[] = {
  { "STRING",     0, TARGET_STRING },
  { "text/plain", 0, TARGET_STRING },
  { "application/x-rootwin-drop", 0, TARGET_ROOTWIN }
};


AlphaKaraoke::AlphaKaraoke(PlayerInterface *player) : KaraokeBase(player)
{
  _backPixmap = NULL;

#ifdef MIDNIGHT_USE_GTK
  _fontName = getGUIPlayer()->getFeature("/gmidnight/karaoke/font_name", "-*-courier-*-*-*-*-24-*-*-*-*-*-*-*");
#else
  _fontName = gnome_config_get_string("/gmidnight/karaoke/font_name=-*-courier-*-*-*-*-24-*-*-*-*-*-*-*");
#endif

  _font = NULL;

  _mainWindow = gtk_window_new( GTK_WINDOW_TOPLEVEL );
  _isShowing = false;

  GtkWidget *vbox = gtk_vbox_new(FALSE, 0);

  gtk_drag_dest_set (_mainWindow,
		     GTK_DEST_DEFAULT_ALL,
		     target_table, sizeof(target_table) / sizeof(target_table[0])-1, /* no rootwin */
		     (GdkDragAction)(GDK_ACTION_COPY | GDK_ACTION_MOVE));
  
  gtk_signal_connect( GTK_OBJECT(_mainWindow), "drag_data_received",
		      GTK_SIGNAL_FUNC( guiplayer_data_dropped ), getGUIPlayer());


  {
    _drawArea = gtk_drawing_area_new();
    gtk_widget_set_usize( GTK_WIDGET (_drawArea), 200, 200);
    _areaWidth = 200;
    _areaHeight = 200;
    gtk_box_pack_start( GTK_BOX(vbox), _drawArea, TRUE, TRUE, 0);
    gtk_widget_show(_drawArea);
  }

  {
    GtkWidget* hbox = gtk_hbox_new(FALSE, 10);

    _statusBar = gtk_statusbar_new();
    gtk_box_pack_start (GTK_BOX (hbox), _statusBar, TRUE, TRUE, 0);
    gtk_widget_show (_statusBar);
    _statusBarContextId = gtk_statusbar_get_context_id(GTK_STATUSBAR(_statusBar), "Statusbar");

    /*
      GtkWidget* prefs = gtk_button_new_with_label("prefs");
      gtk_signal_connect( GTK_OBJECT(prefs), "clicked",
      GTK_SIGNAL_FUNC( alphakara_prefs ), this);

      gtk_box_pack_start( GTK_BOX(hbox), prefs, FALSE, FALSE, 6);
      gtk_widget_show(prefs);
    */
    gtk_widget_show(hbox);
    gtk_box_pack_start( GTK_BOX( vbox ), hbox, FALSE, FALSE, 0 );
  }

  gtk_widget_show(vbox);
  gtk_container_add( GTK_CONTAINER(_mainWindow), vbox);


  gtk_signal_connect( GTK_OBJECT(_drawArea), "configure_event",
		      GTK_SIGNAL_FUNC( alphakara_configure_event ), this);
  gtk_signal_connect( GTK_OBJECT(_drawArea), "expose_event",
		      GTK_SIGNAL_FUNC( alphakara_expose_event ), this);
  gtk_signal_connect( GTK_OBJECT(_mainWindow), "delete_event",
		      GTK_SIGNAL_FUNC( alphakara_delete_event ), this);

#ifdef MIDNIGHT_USE_GTK
  _colorBg.red = getGUIPlayer()->getIntFeature("/gmidnight/karaoke_bg_color/red", 65535);
  _colorBg.green = getGUIPlayer()->getIntFeature("/gmidnight/karaoke_bg_color/green",65535);
  _colorBg.blue =  getGUIPlayer()->getIntFeature("/gmidnight/karaoke_bg_color/blue", 65535);
#else
  _colorBg.red = gnome_config_get_int("/gmidnight/karaoke_bg_color/red=65535");
  _colorBg.green = gnome_config_get_int("/gmidnight/karaoke_bg_color/green=65535");
  _colorBg.blue =  gnome_config_get_int("/gmidnight/karaoke_bg_color/blue=65535");
#endif

#ifdef MIDNIGHT_USE_GTK
  _colorFgRead.red = getGUIPlayer()->getIntFeature("/gmidnight/karaoke_fg_read_color/red", 32767);
  _colorFgRead.green = getGUIPlayer()->getIntFeature("/gmidnight/karaoke_fg_read_color/green", 32767);
  _colorFgRead.blue = getGUIPlayer()->getIntFeature("/gmidnight/karaoke_fg_read_color/blue", 32767);
#else
  _colorFgRead.red = gnome_config_get_int("/gmidnight/karaoke_fg_read_color/red=32767");
  _colorFgRead.green = gnome_config_get_int("/gmidnight/karaoke_fg_read_color/green=32767");
  _colorFgRead.blue = gnome_config_get_int("/gmidnight/karaoke_fg_read_color/blue=32767");
#endif

#ifdef MIDNIGHT_USE_GTK
  _colorFgCurRead.red = getGUIPlayer()->getIntFeature("/gmidnight/karaoke_fg_cur_color/red", 65535);
  _colorFgCurRead.green =  getGUIPlayer()->getIntFeature("/gmidnight/karaoke_fg_cur_color/green", 0);
  _colorFgCurRead.blue = getGUIPlayer()->getIntFeature("/gmidnight/karaoke_fg_cur_color/blue", 0);
#else
  _colorFgCurRead.red = gnome_config_get_int("/gmidnight/karaoke_fg_cur_color/red=65535");
  _colorFgCurRead.green =  gnome_config_get_int("/gmidnight/karaoke_fg_cur_color/green=0");
  _colorFgCurRead.blue = gnome_config_get_int("/gmidnight/karaoke_fg_cur_color/blue=0");
#endif

#ifdef MIDNIGHT_USE_GTK
  _colorFgUnread.red = getGUIPlayer()->getIntFeature("/gmidnight/karaoke_fg_unread_color/red", 0);
  _colorFgUnread.green = getGUIPlayer()->getIntFeature("/gmidnight/karaoke_fg_unread_color/green", 0);
  _colorFgUnread.blue = getGUIPlayer()->getIntFeature("/gmidnight/karaoke_fg_unread_color/blue", 0);
#else
  _colorFgUnread.red = gnome_config_get_int("/gmidnight/karaoke_fg_unread_color/red=0");
  _colorFgUnread.green = gnome_config_get_int("/gmidnight/karaoke_fg_unread_color/green=0");
  _colorFgUnread.blue = gnome_config_get_int("/gmidnight/karaoke_fg_unread_color/blue=0");
#endif

  gdk_color_alloc( gdk_colormap_get_system(), &_colorBg);
  gdk_color_alloc( gdk_colormap_get_system(), &_colorFgRead);
  gdk_color_alloc( gdk_colormap_get_system(), &_colorFgUnread);
  gdk_color_alloc( gdk_colormap_get_system(), &_colorFgCurRead);

  _lineNumber = -1;
  _topLineIndex = 0;
  _lastTick = 0;
  _currentLineIndex = -1;
  _yoffset = 0;

  _scrollType = SCROLL_CENTER;
  _scrollSmooth = SCROLL_SMOOTH;

  _millisBetweenScrollRefresh = 33;

#ifdef MIDNIGHT_USE_GTK
  if (getGUIPlayer()->getFeature("/gmidnight/scroll/smooth","false") != "false")
    _scrollSmooth = SCROLL_SMOOTH;
  else 
    _scrollSmooth = SCROLL_LINE;
#else
  if (gnome_config_get_bool("/gmidnight/scroll/smooth=false"))
    _scrollSmooth = SCROLL_SMOOTH;
  else 
    _scrollSmooth = SCROLL_LINE;
#endif

#ifdef MIDNIGHT_USE_GTK
  _scrollType = getGUIPlayer()->getIntFeature("/gmidnight/scroll/type", 0);
#else
  _scrollType = gnome_config_get_int("/gmidnight/scroll/type=0");
#endif

  gtk_widget_realize( _mainWindow );
  gtk_widget_realize( _drawArea );

  setFont( getFontName(), true);
}

void AlphaKaraoke::computeLines()
{
  //  cerr << "in compute lines " << _font << endl;
  if (_font == NULL)
    {
      //      cerr << " ???? return:!" << endl;
      return;
    }
  //  cerr << "heree ! with area " << _areaWidth << "," << _areaHeight <<  endl;

  vector<pair<unsigned long, string> >::iterator iter = getEventBegin();
  vector<pair<unsigned long, string> >::iterator end = getEventEnd();

  _lyricsLines.clear();

  //
  // fill-in the line info vector

  LineInfo skip;
  skip.first = end;
  skip.end = end;
  skip.skip = true;
  skip.right = false;

      
  LineInfo curline;
  curline.first = iter;
  curline.right = false;
  curline.text = "";
  bool crlf = true;
  int lyridx = 0;
  int lyrInARow = 0;
  for (int curwidth=0; iter != end; )
    {
      if (crlf == true)
	{
	  //	  cerr << "first = " << iter->second << endl;
	  crlf = false;
	  curline.first = iter;
	  curline.end = iter;
	  curline.skip = false;
	  lyrInARow = 0;
	  curwidth = 0;
	  lyridx = 0;

	  if ((iter->second.find('\\') != string::npos))
	    {
	      //	      _lyricsLines.push_back(skip);
	    }
	}

      int lyrwidth;
      gdk_string_extents(_font,
			 strippedCharArrayFromString(iter->second),
			 NULL, NULL,
			 &lyrwidth, NULL, NULL);
      curwidth += lyrwidth;

      if ((iter+1) != end)
	{
	  if (((iter+1)->second.find('/') != string::npos)
	      || ((iter+1)->second.find('\\') != string::npos))
	    {
	      crlf = true;
	    }
	}
      
      if (curwidth > _areaWidth)
	{
	  if (lyrInARow == 0)
	    curline.end = ++iter;
	  else
	    curline.end = iter;
	  _lyricsLines.push_back(curline);
	  crlf = true;
	  curline.skip = false;
	  curline.right = true;

	  //	  cerr << "end = " << iter->second << endl;
	  //	  iter++;
	}
      else if ((iter->second.find('\n') != string::npos)
	       || (iter->second.find('\r') != string::npos)
	       || crlf)
	{
	  curline.end = iter+1;
	  _lyricsLines.push_back(curline);
	  crlf = true;
	  curline.skip = false;
	  curline.right = false;

	  //	  cerr << "end = " << iter->second << endl;
	  iter++;
	}
      else
	{
	  //	  cerr << "ignoring " << iter->second << endl;
	  iter ++;
	  lyrInARow++;
	}
    }
  
  if (curline.first != iter)
    {
      curline.end = end;
      _lyricsLines.push_back(curline);
    }

  //
  // Computes the LineInfo.text variable
  //
  for (unsigned int i=0; i<_lyricsLines.size(); i++)
    {
      LineInfo &lit = _lyricsLines[i];
      string s = "";
      vector<pair<unsigned long, string> >::iterator cur = lit.first;
      for (; cur != lit.end; cur++)
	{
	  s += cur->second;
	}

      lit.text = s;
    }
}


void AlphaKaraoke::draw(GdkEventExpose *event)
{
  //  cerr << "in Draw _yoffset=" << _yoffset << " & topline=" << _topLineIndex 
  //       << " , remaining=" << _scrollRemaining << endl;

  if (_backPixmap == NULL)
    updatePixmapBuffer();

  for (int i=0; i<_lineNumber; i++)
    {
      //      if ((curv > (event->area.y-_fontHeight)) && (curv<event->area.y))
      gdk_draw_pixmap(_backPixmap,
		      _drawArea->style->fg_gc[GTK_WIDGET_STATE(_drawArea)],
		      _linesPixmap[i],
		      0,0,
		      0, (i*_lineHeight) - _yoffset,
		      _drawArea->allocation.width, _lineHeight);
    }

  gdk_draw_pixmap(_drawArea->window,
		  _drawArea->style->fg_gc[GTK_WIDGET_STATE(_drawArea)],
		  _backPixmap,
		  event->area.x, event->area.y, 
		  event->area.x, event->area.y,
		  event->area.width, event->area.height);
}

void AlphaKaraoke::updatePixmapBuffer()
{
  //  cerr << "in updatePixmapBuffer" << endl;

  if (_backPixmap != NULL)
    {
      gdk_pixmap_unref( _backPixmap );
    }

  //  cerr << "window updated to " << _drawArea->allocation.width << "," << _drawArea->allocation.height << endl;

  _backPixmap = gdk_pixmap_new( _drawArea->window,
				_drawArea->allocation.width,
				_drawArea->allocation.height,
				-1);

  gdk_draw_rectangle(_backPixmap, _drawArea->style->white_gc,
		     TRUE, 0,0,
		     _drawArea->allocation.width, _drawArea->allocation.height);

  _areaWidth = _drawArea->allocation.width;
  _areaHeight = _drawArea->allocation.height;

  
  if (_lineNumber>=0)
    {
      for (int i=0; i<_lineNumber; i++)
	gdk_pixmap_unref( _linesPixmap[i] );
      delete[] _linesPixmap;
    }

  _lineNumber = (_drawArea->allocation.height / (_lineHeight+1)) + 2;
  _linesPixmap = new (GdkPixmap*)[_lineNumber];

  for (int i=0; i<_lineNumber; i++)
    {
      _linesPixmap[i] = gdk_pixmap_new( _drawArea->window,
					_drawArea->allocation.width,
					(_lineHeight+1),
					-1);
    }

  computeLines();
  //  fillLinesPixmaps();
  //  _topLineIndex = _currentLineIndex = findLineForTick(_lastTick);

  _currentLineIndex = findLineForTick(_lastTick);

  _topLineIndex = defaultTopLineForCurrent(_currentLineIndex);

  redrawAllLinesPixmaps();
  ticked(_lastTick);
}

void AlphaKaraoke::updateLinesPixmap()
{
  if (_lineNumber<0)
    {
      cerr << "invalid AlphaKaraoke::_lineNumber @ updateLinesPixmap" << endl;
      return;
    }
  //////////////////////////////
  for (int i=0; i<_lineNumber; i++)
    {
      //      if ((i+_topLineIndex) < _lineNumber
    }
}

void AlphaKaraoke::fillLinesPixmaps()
{
  GdkGC *curgc = gdk_gc_new(_mainWindow->window);
  gdk_gc_copy(curgc, _drawArea->style->black_gc);
  gdk_gc_set_foreground(curgc, &_colorFgRead);

  for (unsigned int curl = _topLineIndex; (curl<(unsigned int)(_topLineIndex+_lineNumber)); curl++ )
    {
      
      gdk_draw_rectangle(_linesPixmap[curl-_topLineIndex], _drawArea->style->white_gc,
			 TRUE, 0,0,
			 _drawArea->allocation.width, _lineHeight);
	  
      if (curl < _lyricsLines.size())
	{
	  int xoffset;
	  gdk_string_extents(_font,
			     strippedCharArrayFromString(_lyricsLines[curl].text),
			     NULL, NULL,
			     &xoffset, NULL, NULL);
	  xoffset = (_areaWidth - xoffset) / 2;

	  gdk_draw_string( _linesPixmap[curl-_topLineIndex], 
			   _font,
			   curgc, //			   _drawArea->style->black_gc,
			   //			   5, curh + _lineInfo[curl].Ascent,
			   xoffset, _fontAscent,
			   strippedCharArrayFromString(_lyricsLines[curl].text));

	  //	  cerr << "put \""<< _lyricsLines[curl].text << "\" in line pixmap " << (curl) << endl;
	}
    }

}

void AlphaKaraoke::fillLinesPixmaps(int line, int pixline, bool sang)
{
  GdkGC *curgc = gdk_gc_new(_mainWindow->window);
  gdk_gc_copy(curgc, _drawArea->style->black_gc);  

  gdk_gc_set_foreground(curgc, &_colorBg);
  gdk_draw_rectangle(_linesPixmap[pixline], curgc,
		     TRUE, 0,0,
		     _drawArea->allocation.width, _lineHeight);

  if (sang)
    {
      gdk_gc_set_foreground(curgc, &_colorFgRead);
    }
  else
    {
      gdk_gc_set_foreground(curgc, &_colorFgUnread);      
    }



  if (((unsigned int)line < _lyricsLines.size()) && (line >= 0))
    {
      if (_lyricsLines[line].skip == false)
	{
	  int xoffset;
	  string strval = trimString(_lyricsLines[line].text);
	  gdk_string_extents(_font,
			     strippedCharArrayFromString(strval),
			     NULL, NULL,
			     &xoffset, NULL, NULL);
	  xoffset = (_areaWidth - xoffset) / 2;
	  gdk_draw_string( _linesPixmap[pixline], 
			   _font,
			   curgc, //			   _drawArea->style->black_gc,
			   //			   5, curh + _lineInfo[curl].Ascent,
			   xoffset, _fontAscent,
			   strippedCharArrayFromString(strval));
	}
    }
  gdk_gc_unref(curgc);
}

void AlphaKaraoke::fillCurrentLinePixmap(int line, int pixline, unsigned int currentTick)
{
  GdkGC *curgc = gdk_gc_new(_mainWindow->window);
  gdk_gc_copy(curgc, _drawArea->style->black_gc);

  gdk_gc_set_foreground(curgc, &_colorBg);
  gdk_draw_rectangle(_linesPixmap[pixline], curgc,
		     TRUE, 0,0,
		     _drawArea->allocation.width, _lineHeight);

  gdk_gc_set_foreground(curgc, &_colorFgRead);  

  if (line < (int)_lyricsLines.size())
    {

      LineInfo &linfo = _lyricsLines[line];

      int xoffset;
      int lyrwidth;
      string strval = trimString(linfo.text);
      gdk_string_extents(_font,
			 strippedCharArrayFromString(strval),
			 NULL, NULL,
			 &lyrwidth, NULL, NULL);
      xoffset = (_areaWidth - lyrwidth) / 2;

      bool firstPass = true;
      bool highlight_done = false;
      for (vector<pair<unsigned long, string> >::iterator lyr = linfo.first; lyr != linfo.end; lyr++)
	{
	  if (lyr->first > currentTick)
	    gdk_gc_set_foreground(curgc, &_colorFgUnread);
	  else if (lyr->first == currentTick)
	    {
	      gdk_gc_set_foreground(curgc, &_colorFgCurRead);
	      highlight_done = true;
	    }
	  else if (((lyr+1)!=linfo.end) && (highlight_done == false))
	    {
	      if ((currentTick > lyr->first) && (currentTick < ((lyr+1)->first)))
		{
		  gdk_gc_set_foreground(curgc, &_colorFgCurRead);
		  highlight_done = true;
		}
	    }
	  else if (((lyr+1)==linfo.end) && (highlight_done == false))
	    {
	      gdk_gc_set_foreground(curgc, &_colorFgCurRead);
	    }

	  string val;
	  if (firstPass)
	    {
	      val = trimStringBegin(lyr->second);
	      firstPass = false;
	    }
	  else val = lyr->second;

	  gdk_string_extents(_font,
			     strippedCharArrayFromString(val),
			     NULL, NULL,
			     &lyrwidth, NULL, NULL);

	  gdk_draw_string(_linesPixmap[pixline], _font, curgc,
			  xoffset, _fontAscent, 
			  strippedCharArrayFromString(val));
	  xoffset += lyrwidth;
	}
      //      cerr<< "current line drawn : " << linfo.text << endl;
    }
  gdk_gc_unref(curgc);
}

void AlphaKaraoke::ticked(int tick)
{
  if ( gdk_window_is_visible(GTK_WIDGET(_mainWindow)->window) == FALSE)
    return;

  _lastTick = tick;
  int nuline = findLineForTick(tick);

  if (_currentLineIndex != nuline)
    {
      if (_scrollSmooth == SCROLL_LINE)
	{
	  switch (_scrollType)
	    {
	    case SCROLL_CENTER:
	      _topLineIndex = nuline - (_lineNumber/2) +1;
	      _currentLineIndex = nuline;
	      break;

	    case SCROLL_TOP:
	      _topLineIndex = nuline-1;
	      _currentLineIndex = nuline;
	      break;

	    case SCROLL_FRAMED:
	      if (nuline >= (_topLineIndex+ ((_lineNumber*2)/3)))
		{
		  if (_lineNumber == 1)
		    _topLineIndex = nuline;
		  else _topLineIndex = nuline-1;
		}
	      //	      _currentLineIndex = nuline;
	      break;
	    }
	}
      else
	{
	  int nextline = nuline+1;

	  switch (_scrollType)
	    {
	    case SCROLL_CENTER:
	      _topLineIndex = nuline - (_lineNumber/2) + 1;
	      _currentLineIndex = nuline;
	      if (nuline < (int)_lyricsLines.size())
		{
		  if ((_yoffset>0) && (_yoffset!=_scrollRemaining))
		    _yoffsetError = _scrollRemaining - _yoffset;
		  else _yoffsetError = 0;
		  _yoffset = -_yoffsetError;
		  startScrolling(ticksToMs(_lyricsLines[nextline].first->first-_lastTick), _lineHeight );
		}
	      break;

	    case SCROLL_TOP:
	      _topLineIndex = nuline-2;
	      _currentLineIndex = nuline;
	      if (nuline < (int)_lyricsLines.size())
		{
		  _yoffset = 0;
		  if ((_yoffset>0) && (_yoffset!=_scrollRemaining))
		    _yoffsetError = _scrollRemaining - _yoffset;
		  else 
		    _yoffsetError = 0;
		  startScrolling(ticksToMs(_lyricsLines[nextline].first->first-_lastTick), _lineHeight);
		}
	      break;

	    case SCROLL_FRAMED:
	      if (((nuline) >= (_topLineIndex+ ((_lineNumber*2)/3))) && (_pageLineUp==0))
		{
		  _yoffset = 0;
		  ++_topLineIndex;
		  //		      startScrolling(ticksToMs(_lyricsLines[nextline].first->first-_lastTick), _lineHeight * ((_lineNumber*2)/3) );
		  _pageLineUp = ((_lineNumber*2)/3) - 2;
		  startScrolling(200, _lineHeight);
		  //		      if (_lineNumber == 1)
		  //			_topLineIndex = nuline;
		  //		      else _topLineIndex = nuline-1;
		}
	      _currentLineIndex = nuline;
	      break;
	    }
	  //	      _topLineIndex = _currentLineIndex;
	  //	      _currentLineIndex = nuline;
	}
      redrawAllLinesPixmaps();
    }
  else if (_lyricsLines.size() > 0)
    {
      if (_lyricsLines[_currentLineIndex].first != _lyricsLines[_currentLineIndex].end)
	{
	  if (((unsigned int)_lastTick == (_lyricsLines[_currentLineIndex].end-1)->first) && (_scrollType == SCROLL_FRAMED))
	    {
	      if (_scrollSmooth == SCROLL_SMOOTH)
		{

		}
	      else 
		{
		  _currentLineIndex = nuline;
		}
	    }
	}
      redrawCurrentLinePixmap();
    }

}

int AlphaKaraoke::defaultTopLineForCurrent(int curline)
{
  if (_scrollSmooth == SCROLL_LINE)
    {
      switch (_scrollType)
	{
	case SCROLL_CENTER:
	  return curline - (_lineNumber/2)+2;

	case SCROLL_TOP:
	  return  curline-1;

	case SCROLL_FRAMED:
	  return curline;
	}
    }  
  else
    {
      switch (_scrollType)
	{
	case SCROLL_CENTER:
	  return curline - (_lineNumber/2)+2;

	case SCROLL_TOP:
	  return  curline-1;

	case SCROLL_FRAMED:
	  return  curline;
	  
	}
    }
  return curline;
}

int AlphaKaraoke::findLineForTick(unsigned int tick)
{
  for (unsigned int i=0; i<_lyricsLines.size(); i++)
    {
      LineInfo &linfo = _lyricsLines[i];
      if ((tick >= linfo.first->first) && (tick < linfo.end->first))
	return i;
    }
  return 0;
}

void AlphaKaraoke::redrawCurrentLinePixmap()
{
  int curl = _currentLineIndex - _topLineIndex;
  if (curl < 0)
    {
      redrawAllLinesPixmaps();
      return;
    }
  fillCurrentLinePixmap(_currentLineIndex, _currentLineIndex - _topLineIndex, _lastTick);

  GdkRectangle updrect;
  updrect.x = 0;
  updrect.y = 0;
  updrect.width = _drawArea->allocation.width-1;
  updrect.height = _drawArea->allocation.height-1;
  
  gtk_widget_draw(_drawArea, &updrect);
}

void AlphaKaraoke::redrawAllLinesPixmaps()
{
  //  cerr << "redraw all lines pixmaps (cur=" << _currentLineIndex << ")(top=" << _topLineIndex << ")" << endl;
  for (int i = _topLineIndex; i < (_topLineIndex+_lineNumber); i++)
    {
      if (i < _currentLineIndex)
	fillLinesPixmaps(i, i - _topLineIndex, true);
      else if (i == _currentLineIndex)
	fillCurrentLinePixmap(i, i - _topLineIndex, _lastTick);
      else
	fillLinesPixmaps(i, i - _topLineIndex, false);
    }

  GdkRectangle updrect;
  updrect.x = 0;
  updrect.y = 0;
  updrect.width = _drawArea->allocation.width-1;
  updrect.height = _drawArea->allocation.height-1;
  
  gtk_widget_draw(_drawArea, &updrect);
}

const char *AlphaKaraoke::strippedCharArrayFromString(string &str)
{
  if ((str[0] == '\\') || (str[0]=='/'))
    {
      return &str.c_str()[1];
    }
  return str.c_str();
}

string AlphaKaraoke::trimStringBegin(const string &str)
{
  int i = 0;
  while (str[i] == ' ')
    i++;
  if (i==0)
    return str;
  
  return str.substr(i);
}
string AlphaKaraoke::trimString(const string &str)
{
  int beg = 0;
  while (str[beg] == ' ')
    beg++;

  int end = str.length()-1;
  while (str[end] == ' ')
    end--;

  if (beg >= end)
    return str;

  return str.substr(beg, end-beg+1);
}

bool AlphaKaraoke::isShowing() 
{ 
  return _isShowing;
}

void AlphaKaraoke::show() 
{
  _isShowing = true;
  gtk_widget_show(_mainWindow);
  if (getGUIPlayer() != 0)
    {
      getGUIPlayer()->toggleMenuItem_show_karaoke(true);
    }
}

void AlphaKaraoke::hide() 
{
  _isShowing = false;
  gtk_widget_hide(_mainWindow);
  if (getGUIPlayer() != 0)
    {
      getGUIPlayer()->toggleMenuItem_show_karaoke(false);
    }
}

void AlphaKaraoke::showPreferences()
{
  ALPHAKARAOKEPREFSCLASS* prefs = new ALPHAKARAOKEPREFSCLASS(this);
  
  //  cerr << "NEW PREFS CREATED!" << endl;
  prefs->show();

}

string& AlphaKaraoke::getFontName()
{
  return _fontName;
}
void AlphaKaraoke::setFont(string font, bool save)
{
  if (_font != NULL)
    gdk_font_unref(_font);

  _fontName = font;
  _font = gdk_font_load( font.c_str() );
  gdk_string_extents(_font,
		     "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ",
		     NULL, NULL,
		     NULL, &_fontAscent, &_fontDescent);
  _fontHeight = _fontDescent + _fontAscent;
  _lineHeight = _fontHeight*2;
  
#ifdef MIDNIGHT_USE_GTK
  getGUIPlayer()->setFeature("/gmidnight/karaoke/font_name", font.c_str());
  getGUIPlayer()->flushConfiguration();
#else
  gnome_config_set_string("/gmidnight/karaoke/font_name", font.c_str());
  gnome_config_sync();
#endif

  updatePixmapBuffer();
}


void AlphaKaraoke::ReallocColor(GdkColor*col, int r, int g, int b)
{
  //  gdk_color_free(col);
  col->red = r;
  col->green = g;
  col->blue = b;
  gdk_color_alloc(gdk_colormap_get_system(), col);
}

void AlphaKaraoke::setReadForegroundColor(int r, int g, int b)
{
  ReallocColor(&_colorFgRead,r,g,b);
#ifdef MIDNIGHT_USE_GTK
  getGUIPlayer()->setIntFeature("/gmidnight/karaoke_fg_read_color/red",r);
  getGUIPlayer()->setIntFeature("/gmidnight/karaoke_fg_read_color/green",g);
  getGUIPlayer()->setIntFeature("/gmidnight/karaoke_fg_read_color/blue",b);
  getGUIPlayer()->flushConfiguration();

#else
  gnome_config_set_int("/gmidnight/karaoke_fg_read_color/red",r);
  gnome_config_set_int("/gmidnight/karaoke_fg_read_color/green",g);
  gnome_config_set_int("/gmidnight/karaoke_fg_read_color/blue",b);
  gnome_config_sync();
#endif
}
GdkColor* AlphaKaraoke::getReadForegroundColor() { return &_colorFgRead; }

void AlphaKaraoke::setUnreadForegroundColor(int r, int g, int b)
{
  ReallocColor(&_colorFgUnread,r,g,b);
#ifdef MIDNIGHT_USE_GTK
  getGUIPlayer()->setIntFeature("/gmidnight/karaoke_fg_unread_color/red",r);
  getGUIPlayer()->setIntFeature("/gmidnight/karaoke_fg_unread_color/green",g);
  getGUIPlayer()->setIntFeature("/gmidnight/karaoke_fg_unread_color/blue",b);
  getGUIPlayer()->flushConfiguration();
#else
  gnome_config_set_int("/gmidnight/karaoke_fg_unread_color/red",r);
  gnome_config_set_int("/gmidnight/karaoke_fg_unread_color/green",g);
  gnome_config_set_int("/gmidnight/karaoke_fg_unread_color/blue",b);
  gnome_config_sync();
#endif
}
GdkColor* AlphaKaraoke::getUnreadForegroundColor() { return &_colorFgUnread; }

void AlphaKaraoke::setCurrentForegroundColor(int r, int g, int b)
{
  ReallocColor(&_colorFgCurRead,r,g,b);
#ifdef MIDNIGHT_USE_GTK
  getGUIPlayer()->setIntFeature("/gmidnight/karaoke_fg_cur_color/red",r);
  getGUIPlayer()->setIntFeature("/gmidnight/karaoke_fg_cur_color/green",g);
  getGUIPlayer()->setIntFeature("/gmidnight/karaoke_fg_cur_color/blue",b);
  getGUIPlayer()->flushConfiguration();
#else
  gnome_config_set_int("/gmidnight/karaoke_fg_cur_color/red",r);
  gnome_config_set_int("/gmidnight/karaoke_fg_cur_color/green",g);
  gnome_config_set_int("/gmidnight/karaoke_fg_cur_color/blue",b);
  gnome_config_sync();
#endif
}

GdkColor* AlphaKaraoke::getCurrentForegroundColor()  
{ 
  return &_colorFgCurRead; 
}


void AlphaKaraoke::setBackgroundColor(int r, int g, int b)
{
  ReallocColor(&_colorBg,r,g,b);
#ifdef MIDNIGHT_USE_GTK
  getGUIPlayer()->setIntFeature("/gmidnight/karaoke_bg_color/red",r);
  getGUIPlayer()->setIntFeature("/gmidnight/karaoke_bg_color/green",g);
  getGUIPlayer()->setIntFeature("/gmidnight/karaoke_bg_color/blue",b);
  getGUIPlayer()->flushConfiguration();
#else
  gnome_config_set_int("/gmidnight/karaoke_bg_color/red",r);
  gnome_config_set_int("/gmidnight/karaoke_bg_color/green",g);
  gnome_config_set_int("/gmidnight/karaoke_bg_color/blue",b);
  gnome_config_sync();
#endif
}
GdkColor* AlphaKaraoke::getBackgroundColor()   { return &_colorBg; }

void AlphaKaraoke::setScrollType(int smooth, int type)
{
  _scrollType = type;

#ifdef MIDNIGHT_USE_GTK
  getGUIPlayer()->setIntFeature("/gmidnight/scroll/type",type);
#else
  gnome_config_set_int("/gmidnight/scroll/type",type);
#endif

  _scrollSmooth = smooth;
#ifdef MIDNIGHT_USE_GTK
  getGUIPlayer()->setFeature("/gmidnight/scroll/smooth",(smooth==SCROLL_SMOOTH)?"true":"false");
  getGUIPlayer()->flushConfiguration();
#else
  gnome_config_set_bool("/gmidnight/scroll/smooth",(smooth==SCROLL_SMOOTH)?TRUE:FALSE);
  gnome_config_sync();
#endif
  updatePixmapBuffer();

  update(_lastTick);
}

bool AlphaKaraoke::scrollIsSmooth()
{
  return (_scrollSmooth==SCROLL_SMOOTH)?true:false;
}

int AlphaKaraoke::getScrollType()
{
  return _scrollType;
}

void AlphaKaraoke::update(unsigned int ticks)
{
  //  cerr << endl << endl << "-----------------------\nUPDATE!!\n-----------------\n";
  _currentLineIndex = -2;
  _yoffset = 0;
  _yoffsetError = 0;
  if (_scrollTimeOutID != 0)
    {
      gtk_timeout_remove(_scrollTimeOutID);
      _scrollTimeOutID = 0;
    }

  updatePixmapBuffer();
  ticked(ticks);
}

unsigned int AlphaKaraoke::timevalDiffMillis(timeval &first, timeval &second)
{
  unsigned int secsMillis = (second.tv_sec - first.tv_sec) * 1000;
  unsigned int usecsMillis = (second.tv_usec - first.tv_usec) / 1000;
  
  return secsMillis + usecsMillis;
}

void AlphaKaraoke::startScrolling(int ms, int pixels)
{
  gettimeofday(&_scrollTimeStartedAt, NULL);
  _scrollTimeOutExpiresAt = _scrollTimeStartedAt;
  _scrollTimeOutExpiresAt.tv_sec += (ms/1000); // TODO: probably have to check that
  _scrollTimeOutExpiresAt.tv_usec += ((ms%1000)*1000); // TODO: probably have to check that

  //  cerr << "[[[[[[]]]]]] EXPIRES " << _scrollTimeOutExpiresAt.tv_usec << endl;
  
  _scrollRemaining = pixels;
  _scrollTimeSlice = ms/pixels;

  //  gettimeofday(&_scrollTimeOutLast, NULL);
  _scrollTimeTotalMillis = timevalDiffMillis(_scrollTimeStartedAt, _scrollTimeOutExpiresAt);

  if (_scrollTimeOutID == 0)
    {
      _scrollTimeOutID = gtk_timeout_add(_millisBetweenScrollRefresh, alphakara_scroll_timeout, this);
    }
}


int AlphaKaraoke::scrollArea()
{
  timeval curtime;
  gettimeofday(&curtime, NULL);
  unsigned int millisSinceFirst = timevalDiffMillis(_scrollTimeStartedAt, curtime);

  _yoffset = ((millisSinceFirst * (_scrollRemaining+_yoffsetError)) / _scrollTimeTotalMillis) - _yoffsetError;
      
  if (_yoffset > _scrollRemaining)
    _yoffset = _scrollRemaining;
      
  if ((_scrollSmooth == SCROLL_SMOOTH) && (_scrollType == SCROLL_FRAMED) && (_yoffset == _scrollRemaining) && (_pageLineUp>0))
    {
      //      _yoffset = 0;
	
      if ((_yoffset>0) && (_yoffset!=_scrollRemaining))
	_yoffsetError = _scrollRemaining - _yoffset;
      else _yoffsetError = 0;
      _yoffset = -_yoffsetError;

      --_pageLineUp;
      _topLineIndex++;
      redrawAllLinesPixmaps();
      _scrollTimeStartedAt = curtime;
      _scrollTimeOutExpiresAt = _scrollTimeStartedAt;
      _scrollTimeOutExpiresAt.tv_usec += 1000*200;
      _scrollTimeTotalMillis = timevalDiffMillis(_scrollTimeStartedAt, _scrollTimeOutExpiresAt);
    }
  else 
    if (_previousYOffset != _yoffset)
      {
	GdkRectangle updrect;
	updrect.x = 0;
	updrect.y = 0;
	updrect.width = _drawArea->allocation.width-1;
	updrect.height = _drawArea->allocation.height-1;
	gtk_widget_draw(_drawArea, &updrect);
	_previousYOffset = _yoffset;
      }
  
  return 0;
}


void AlphaKaraoke::initLyrics()
{
  KaraokeBase::initLyrics();
  if (_scrollTimeOutID != 0)
    {
      gtk_timeout_remove(_scrollTimeOutID);
      _scrollTimeOutID = 0;
    }
  init();
  update(0);
}

void AlphaKaraoke::setStatus(string str)
{
  gtk_statusbar_push(GTK_STATUSBAR(_statusBar), _statusBarContextId, str.c_str());
}
