/*
  Midnight, a C++ midi library to load and play midi files.
  Copyright (C) 1999 Rodrigo Reyes, reyes@chez.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "AlphaKaraoke.h"
#include "gnome.h"

#ifdef MIDNIGHT_USE_GNOME

static gint alphakara_gnomeprefs_set_font(GtkWidget *widget, gchar *fname, gpointer data)
{
  ((AlphaKaraoke::AlphaKaraokeGnomePrefs*)data)->setFont(fname);
  return FALSE;
}

static gint alphakara_gnomeprefs_set_color(GtkWidget* widget, int r, int g, int b, int a, gpointer data)
{
  ((AlphaKaraoke::AlphaKaraokeGnomePrefs*)data)->changed();
  return FALSE;
}

static gint alphakara_gnomeprefs_apply(GnomePropertyBox *property_box, gint page_num, gpointer data)
{
  ((AlphaKaraoke::AlphaKaraokeGnomePrefs*)data)->apply(page_num);
  return FALSE;
}

static gint alphakara_gnomeprefs_changescroll(GtkWidget *widget, gpointer data)
{
  ((AlphaKaraoke::AlphaKaraokeGnomePrefs*)data)->changed();
  return FALSE;
}

static gint alphakara_gnomeprefs_delete_event(GtkWidget* widget, GdkEventConfigure* event, gpointer data)
{
  delete ((AlphaKaraoke::AlphaKaraokeGnomePrefs*)data);
  return TRUE;
}

static gint alphakara_gnomeprefs_cancel(GnomePropertyBox *property_box, gpointer data)
{
  delete ((AlphaKaraoke::AlphaKaraokeGnomePrefs*)data);
  return TRUE;
}

AlphaKaraoke::AlphaKaraokeGnomePrefs::AlphaKaraokeGnomePrefs(AlphaKaraoke*mother)
{
  _mother = mother;
  _main = gnome_property_box_new( );
  gtk_widget_realize(_main);

  GtkWidget *table = gtk_table_new(2,5, TRUE);
  {
    GtkWidget *lab = gtk_label_new("Select font");
    gtk_table_attach_defaults(GTK_TABLE(table), lab, 0,1, 0,1);
    gtk_widget_show(lab);

    _fontSelector = gnome_font_picker_new();
    gnome_font_picker_set_font_name (GNOME_FONT_PICKER(_fontSelector), mother->getFontName().c_str());
    gnome_font_picker_set_title     (GNOME_FONT_PICKER(_fontSelector),
				     "Select the font for the lyrics");
    gtk_signal_connect( GTK_OBJECT(_fontSelector), "font_set",
			GTK_SIGNAL_FUNC( alphakara_gnomeprefs_set_font ), this);

    gtk_widget_show(_fontSelector);
    gtk_table_attach_defaults(GTK_TABLE(table), _fontSelector, 1,2, 0,1);
  }

  {
    GtkWidget *lab = gtk_label_new("Background");
    gtk_table_attach(GTK_TABLE(table), lab, 0,1, 1,2, GTK_FILL, GTK_FILL, 0,0);
    gtk_widget_show(lab);

    _backColor = gnome_color_picker_new();
    gtk_widget_show(_backColor);
    gtk_signal_connect( GTK_OBJECT(_backColor), "color_set",
			GTK_SIGNAL_FUNC( alphakara_gnomeprefs_set_color ), this);
    gtk_table_attach(GTK_TABLE(table), _backColor, 1,2, 1,2, GTK_FILL, GTK_FILL, 0,0);


    GdkColor* c = mother->getBackgroundColor();
    gnome_color_picker_set_i16 (GNOME_COLOR_PICKER(_backColor), c->red, c->green, c->blue, 0);
  }

  {
    GtkWidget *lab = gtk_label_new("Read lyrics");
    gtk_table_attach(GTK_TABLE(table), lab, 0,1, 2,3, GTK_FILL, GTK_FILL, 0,0);
    gtk_widget_show(lab);

    _foreReadColor = gnome_color_picker_new();
    gtk_widget_show(_foreReadColor);
    gtk_signal_connect( GTK_OBJECT(_foreReadColor), "color_set",
			GTK_SIGNAL_FUNC( alphakara_gnomeprefs_set_color ), this);
    gtk_table_attach(GTK_TABLE(table), _foreReadColor, 1,2, 2,3, GTK_FILL, GTK_FILL, 0,0);

    GdkColor* c = mother->getReadForegroundColor();
    gnome_color_picker_set_i16 (GNOME_COLOR_PICKER(_foreReadColor), c->red, c->green, c->blue, 0);
  }

  {
    GtkWidget *lab = gtk_label_new("Unread lyrics");
    gtk_table_attach(GTK_TABLE(table), lab, 0,1, 3,4, GTK_FILL, GTK_FILL, 0,0);
    gtk_widget_show(lab);

    _foreUnreadColor = gnome_color_picker_new();
    gtk_widget_show(_foreUnreadColor);
    gtk_signal_connect( GTK_OBJECT(_foreUnreadColor), "color_set",
			GTK_SIGNAL_FUNC( alphakara_gnomeprefs_set_color ), this);
    gtk_table_attach(GTK_TABLE(table), _foreUnreadColor, 1,2, 3,4, GTK_FILL, GTK_FILL, 0,0);
    GdkColor* c = mother->getUnreadForegroundColor();
    gnome_color_picker_set_i16 (GNOME_COLOR_PICKER(_foreUnreadColor), c->red, c->green, c->blue, 0);
  }

  {
    GtkWidget *lab = gtk_label_new("Highlighted lyrics");
    gtk_table_attach(GTK_TABLE(table), lab, 0,1, 4,5, GTK_FILL, GTK_FILL, 0,0);
    gtk_widget_show(lab);

    _foreCurrentColor = gnome_color_picker_new();
    gtk_widget_show(_foreCurrentColor);
    gtk_signal_connect( GTK_OBJECT(_foreCurrentColor), "color_set",
			GTK_SIGNAL_FUNC( alphakara_gnomeprefs_set_color ), this);
    gtk_table_attach(GTK_TABLE(table), _foreCurrentColor, 1,2, 4,5, GTK_FILL, GTK_FILL, 0,0);

    GdkColor* c = mother->getCurrentForegroundColor();
    gnome_color_picker_set_i16 (GNOME_COLOR_PICKER(_foreCurrentColor), c->red, c->green, c->blue, 0);
  }
  gtk_widget_show(table);
  gnome_property_box_append_page  (GNOME_PROPERTY_BOX(_main),
				   table,
				   gtk_label_new("Display"));


  GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
  GtkWidget *frame = gtk_frame_new("Scroll type");
  GtkWidget *vbox2 = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(frame), vbox2);

  {
    _scrollSmooth = gtk_check_button_new_with_label("Smooth scrolling");
    if (_mother->scrollIsSmooth())
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_scrollSmooth), TRUE);
    else gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_scrollSmooth), FALSE);
    gtk_signal_connect( GTK_OBJECT(_scrollSmooth), "clicked",
			GTK_SIGNAL_FUNC( alphakara_gnomeprefs_changescroll), this);
    gtk_box_pack_start( GTK_BOX(vbox), _scrollSmooth, TRUE, TRUE, 7);
    gtk_widget_show(_scrollSmooth);
  }


  {
    _scrollLineFrame = gtk_radio_button_new_with_label (NULL, "Page jump");
    gtk_signal_connect( GTK_OBJECT(_scrollLineFrame), "clicked",
			GTK_SIGNAL_FUNC( alphakara_gnomeprefs_changescroll), this);
    gtk_box_pack_start( GTK_BOX(vbox2), _scrollLineFrame, TRUE, TRUE, 0);
    gtk_widget_show(_scrollLineFrame);
  }
  {
    _scrollLineTop = gtk_radio_button_new_with_label (gtk_radio_button_group(GTK_RADIO_BUTTON(_scrollLineFrame)), "Keep at top");
    gtk_signal_connect( GTK_OBJECT(_scrollLineTop), "clicked",
			GTK_SIGNAL_FUNC( alphakara_gnomeprefs_changescroll), this);
    gtk_box_pack_start( GTK_BOX(vbox2), _scrollLineTop, TRUE, TRUE, 0);
    gtk_widget_show(_scrollLineTop);
  }
  {
    _scrollLineCenter = gtk_radio_button_new_with_label (gtk_radio_button_group(GTK_RADIO_BUTTON(_scrollLineTop)), "Keep centered");
    gtk_signal_connect( GTK_OBJECT(_scrollLineCenter), "clicked",
			GTK_SIGNAL_FUNC( alphakara_gnomeprefs_changescroll), this);
    gtk_box_pack_start( GTK_BOX(vbox2), _scrollLineCenter, TRUE, TRUE, 0);
    gtk_widget_show(_scrollLineCenter);
  }
  gtk_box_pack_start( GTK_BOX(vbox), frame, TRUE, TRUE, 7);

  switch (_mother->getScrollType())
    {
    case AlphaKaraoke::SCROLL_FRAMED:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_scrollLineFrame), TRUE);
      break;
    case AlphaKaraoke::SCROLL_TOP:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_scrollLineTop), TRUE);
      break;
    case AlphaKaraoke::SCROLL_CENTER:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_scrollLineCenter), TRUE);
      break;
    }
  gtk_widget_show(frame);
  gtk_widget_show(vbox);
  gtk_widget_show(vbox2);
  gnome_property_box_append_page  (GNOME_PROPERTY_BOX(_main),
				   vbox,
				   gtk_label_new("Scrolling"));
  

  gtk_signal_connect( GTK_OBJECT(_main), "apply",
		      GTK_SIGNAL_FUNC( alphakara_gnomeprefs_apply ), this);

  gtk_signal_connect( GTK_OBJECT(_main), "delete_event",
		      GTK_SIGNAL_FUNC( alphakara_gnomeprefs_delete_event ), this);

  gtk_signal_connect( GTK_OBJECT(GNOME_PROPERTY_BOX(_main)->cancel_button), "clicked",
		      GTK_SIGNAL_FUNC( alphakara_gnomeprefs_cancel ), this);

}

void AlphaKaraoke::AlphaKaraokeGnomePrefs::show()
{
  gtk_widget_show(_main);
}

void AlphaKaraoke::AlphaKaraokeGnomePrefs::hide()
{
  gtk_widget_hide(_main);
}

GnomePropertyBox* AlphaKaraoke::AlphaKaraokeGnomePrefs::getPropertyBox()
{
  return GNOME_PROPERTY_BOX(_main);
}

void AlphaKaraoke::AlphaKaraokeGnomePrefs::setFont(char *name)
{
  //cerr << name << " == " << _mother->getFontName() << " ?" << endl;
  if (string(name) != _mother->getFontName())
    {
      gnome_property_box_changed( getPropertyBox());
      //      gnome_font_picker_set_font_name(GNOME_FONT_PICKER(_fontSelector), name);
    }
}

void AlphaKaraoke::AlphaKaraokeGnomePrefs::apply(int page)
{
  switch (page)
    {
    case 0:
      // set the font
      _mother->setFont(string(gnome_font_picker_get_font_name(GNOME_FONT_PICKER(_fontSelector))), true);

      short unsigned int r,g,b,a;

      gnome_color_picker_get_i16(GNOME_COLOR_PICKER(_backColor),&r,&g,&b,&a);
      _mother->setBackgroundColor(r,g,b);

      gnome_color_picker_get_i16(GNOME_COLOR_PICKER(_foreCurrentColor),&r,&g,&b,&a);
      _mother->setCurrentForegroundColor(r,g,b);

      gnome_color_picker_get_i16(GNOME_COLOR_PICKER(_foreReadColor),&r,&g,&b,&a);
      _mother->setReadForegroundColor(r,g,b);

      gnome_color_picker_get_i16(GNOME_COLOR_PICKER(_foreUnreadColor),&r,&g,&b,&a);
      _mother->setUnreadForegroundColor(r,g,b);
      break;
  
    case 1:
      int smooth = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_scrollSmooth))?AlphaKaraoke::SCROLL_SMOOTH:AlphaKaraoke::SCROLL_LINE;
      int type;
      if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_scrollLineFrame)))
	type = AlphaKaraoke::SCROLL_FRAMED;
      else if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_scrollLineCenter)))
	    type = AlphaKaraoke::SCROLL_CENTER;
      else if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_scrollLineTop)))
	    type = AlphaKaraoke::SCROLL_TOP;

      _mother->setScrollType(smooth,type);
    }
  _mother->updatePixmapBuffer();
}

void AlphaKaraoke::AlphaKaraokeGnomePrefs::changed()
{
  gnome_property_box_changed( getPropertyBox());
}



#endif



