/*
  Midnight, a C++ midi library to load and play midi files.
  Copyright (C) 1999 Rodrigo Reyes, reyes@chez.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifdef MIDNIGHT_USE_GTK


#include "AlphaKaraoke.h"

#include "pixmaps/font.xpm"
#include "pixmaps/color.xpm"


static gint alphakara_gtkprefs_set_font(GtkWidget *widget, gpointer data)
{
  ((AlphaKaraoke::AlphaKaraokeGtkPrefs*)data)->showFontDialog();
  return FALSE;
}
static gint alphakara_gtkprefs_set_color(GtkWidget *widget, gpointer data)
{
  ((ColorDialogPrefs*)data)->show();
  return FALSE;
}
static gint alphakara_gtkprefs_ok(GtkWidget *widget, gpointer data)
{
  ((AlphaKaraoke::AlphaKaraokeGtkPrefs*)data)->apply();
  ((AlphaKaraoke::AlphaKaraokeGtkPrefs*)data)->hide();
  return FALSE;
}
static gint alphakara_gtkprefs_cancel(GtkWidget *widget, gpointer data)
{
  delete ((AlphaKaraoke::AlphaKaraokeGtkPrefs*)data);
  return FALSE;
}

static gint alphakara_gtkprefs_delete_event(GtkWidget* widget, GdkEventConfigure* event, gpointer data)
{
  delete ((AlphaKaraoke::AlphaKaraokeGtkPrefs*)data);
  return TRUE;
}


AlphaKaraoke::AlphaKaraokeGtkPrefs::AlphaKaraokeGtkPrefs(AlphaKaraoke*mother)
{
  _mother = mother;
  _main = gtk_dialog_new( );
  gtk_widget_realize(_main);
  
  GtkWidget *notebook = gtk_notebook_new();

  GtkStyle* style = gtk_widget_get_style(_main);

  _colorDialogs = new (ColorDialogPrefs*)[4];

  GdkBitmap* fontbitmask;
  GdkPixmap* fontpix = gdk_pixmap_create_from_xpm_d( _main->window, &fontbitmask, 
						     &style->bg[GTK_STATE_NORMAL], (gchar**)font_xpm);
  GdkPixmap* colbitmask;
  GdkPixmap* colpix = gdk_pixmap_create_from_xpm_d( _main->window, &colbitmask, 
					       &style->bg[GTK_STATE_NORMAL], (gchar**)color_xpm);

  GtkWidget *table = gtk_table_new(2,5, TRUE);
  {
    GtkWidget *lab = gtk_label_new("Select font");
    gtk_table_attach_defaults(GTK_TABLE(table), lab, 0,1, 0,1);
    gtk_widget_show(lab);

    _fontSelector = gtk_button_new();
    GtkWidget*pixmap = gtk_pixmap_new(fontpix, fontbitmask);
    gtk_container_add( GTK_CONTAINER(_fontSelector), pixmap);
    gtk_widget_show(pixmap);

    _fontDialog =  new FontDialogPrefs(mother->getFontName());
    gtk_signal_connect( GTK_OBJECT(_fontSelector), "clicked",
			GTK_SIGNAL_FUNC( alphakara_gtkprefs_set_font ), this);

    gtk_widget_show(_fontSelector);
    gtk_table_attach_defaults(GTK_TABLE(table), _fontSelector, 1,2, 0,1);
  }

  {
    GtkWidget *lab = gtk_label_new("Background");
    gtk_table_attach(GTK_TABLE(table), lab, 0,1, 1,2, GTK_FILL, GTK_FILL, 0,0);
    gtk_widget_show(lab);

    _backColor = gtk_button_new();
    GtkWidget*pixmap = gtk_pixmap_new(colpix, colbitmask);
    gtk_container_add( GTK_CONTAINER(_backColor), pixmap);
    gtk_widget_show(pixmap);
    gtk_widget_show(_backColor);

    GdkColor* c = mother->getBackgroundColor();
    ColorDPUnit cols(c->red, c->green, c->blue);
    _colorDialogs[0] = new ColorDialogPrefs("Background color", cols);
    gtk_signal_connect( GTK_OBJECT(_backColor), "clicked",
    			GTK_SIGNAL_FUNC( alphakara_gtkprefs_set_color ), _colorDialogs[0]);

    gtk_table_attach(GTK_TABLE(table), _backColor, 1,2, 1,2, GTK_FILL, GTK_FILL, 0,0);
  }

  {
    GtkWidget *lab = gtk_label_new("Read lyrics");
    gtk_table_attach(GTK_TABLE(table), lab, 0,1, 2,3, GTK_FILL, GTK_FILL, 0,0);
    gtk_widget_show(lab);

    _foreReadColor = gtk_button_new();
    GtkWidget*pixmap = gtk_pixmap_new(colpix, colbitmask);
    gtk_container_add( GTK_CONTAINER(_foreReadColor), pixmap);
    gtk_widget_show(pixmap);
    gtk_widget_show(_foreReadColor);
    gtk_table_attach(GTK_TABLE(table), _foreReadColor, 1,2, 2,3, GTK_FILL, GTK_FILL, 0,0);


    GdkColor* c = mother->getReadForegroundColor();
    ColorDPUnit cols(c->red, c->green, c->blue);
    _colorDialogs[1] = new ColorDialogPrefs("Read lyrics color", cols);
    gtk_signal_connect( GTK_OBJECT(_foreReadColor), "clicked",
    			GTK_SIGNAL_FUNC( alphakara_gtkprefs_set_color ), _colorDialogs[1]);
  }
  
  {
    GtkWidget *lab = gtk_label_new("Unread lyrics");
    gtk_table_attach(GTK_TABLE(table), lab, 0,1, 3,4, GTK_FILL, GTK_FILL, 0,0);
    gtk_widget_show(lab);

    _foreUnreadColor = gtk_button_new();
    GtkWidget*pixmap = gtk_pixmap_new(colpix, colbitmask);
    gtk_container_add( GTK_CONTAINER(_foreUnreadColor), pixmap);
    gtk_widget_show(pixmap);
    gtk_widget_show(_foreUnreadColor);
    gtk_table_attach(GTK_TABLE(table), _foreUnreadColor, 1,2, 3,4, GTK_FILL, GTK_FILL, 0,0);


    GdkColor* c = mother->getUnreadForegroundColor();
    ColorDPUnit cols(c->red, c->green, c->blue);
    _colorDialogs[2] = new ColorDialogPrefs("Unread lyrics color", cols);
    gtk_signal_connect( GTK_OBJECT(_foreUnreadColor), "clicked",
    			GTK_SIGNAL_FUNC( alphakara_gtkprefs_set_color ), _colorDialogs[2]);
  }
  
  {
    GtkWidget *lab = gtk_label_new("Highlighted lyrics");
    gtk_table_attach(GTK_TABLE(table), lab, 0,1, 4,5, GTK_FILL, GTK_FILL, 0,0);
    gtk_widget_show(lab);

    _foreCurrentColor = gtk_button_new();
    GtkWidget*pixmap = gtk_pixmap_new(colpix, colbitmask);
    gtk_container_add( GTK_CONTAINER(_foreCurrentColor), pixmap);
    gtk_widget_show(pixmap);
    gtk_widget_show(_foreCurrentColor);
    gtk_table_attach(GTK_TABLE(table), _foreCurrentColor, 1,2, 4,5, GTK_FILL, GTK_FILL, 0,0);


    GdkColor* c = mother->getCurrentForegroundColor();
    ColorDPUnit cols(c->red, c->green, c->blue);
    _colorDialogs[3] = new ColorDialogPrefs("Highlited lyrics color", cols);
    gtk_signal_connect( GTK_OBJECT(_foreCurrentColor), "clicked",
    			GTK_SIGNAL_FUNC( alphakara_gtkprefs_set_color ), _colorDialogs[3]);

  }
  

  gtk_widget_show(table);
  gtk_notebook_append_page  (GTK_NOTEBOOK(notebook),
			     table,
			     gtk_label_new("Display"));

  GtkWidget *vbox = gtk_vbox_new(FALSE, 0);
  GtkWidget *frame = gtk_frame_new("Scroll type");
  GtkWidget *vbox2 = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(frame), vbox2);

  {
    _scrollSmooth = gtk_check_button_new_with_label("Smooth scrolling");
    if (_mother->scrollIsSmooth())
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_scrollSmooth), TRUE);
    else gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_scrollSmooth), FALSE);
    gtk_box_pack_start( GTK_BOX(vbox), _scrollSmooth, TRUE, TRUE, 7);
    gtk_widget_show(_scrollSmooth);
  }


  {
    _scrollLineFrame = gtk_radio_button_new_with_label (NULL, "Page jump");
    gtk_box_pack_start( GTK_BOX(vbox2), _scrollLineFrame, TRUE, TRUE, 0);
    gtk_widget_show(_scrollLineFrame);
  }
  {
    _scrollLineTop = gtk_radio_button_new_with_label (gtk_radio_button_group(GTK_RADIO_BUTTON(_scrollLineFrame)), "Keep at top");
    gtk_box_pack_start( GTK_BOX(vbox2), _scrollLineTop, TRUE, TRUE, 0);
    gtk_widget_show(_scrollLineTop);
  }
  {
    _scrollLineCenter = gtk_radio_button_new_with_label (gtk_radio_button_group(GTK_RADIO_BUTTON(_scrollLineTop)), "Keep centered");
    gtk_box_pack_start( GTK_BOX(vbox2), _scrollLineCenter, TRUE, TRUE, 0);
    gtk_widget_show(_scrollLineCenter);
  }
  gtk_box_pack_start( GTK_BOX(vbox), frame, TRUE, TRUE, 7);

  switch (_mother->getScrollType())
    {
    case AlphaKaraoke::SCROLL_FRAMED:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_scrollLineFrame), TRUE);
      break;
    case AlphaKaraoke::SCROLL_TOP:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_scrollLineTop), TRUE);
      break;
    case AlphaKaraoke::SCROLL_CENTER:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(_scrollLineCenter), TRUE);
      break;
    }
  gtk_widget_show(frame);
  gtk_widget_show(vbox);
  gtk_widget_show(vbox2);
  gtk_notebook_append_page  (GTK_NOTEBOOK(notebook),
			     vbox,
			     gtk_label_new("Scrolling"));

  GtkWidget* wid = gtk_button_new_with_label("Cancel");
  gtk_box_pack_end( GTK_BOX( GTK_DIALOG(_main)->action_area ), wid, FALSE, FALSE, 0);
  gtk_signal_connect( GTK_OBJECT(wid), "clicked",
		      GTK_SIGNAL_FUNC( alphakara_gtkprefs_cancel), this);
  gtk_widget_show(wid);
  
  wid = gtk_button_new_with_label("OK");
  gtk_box_pack_end( GTK_BOX( GTK_DIALOG(_main)->action_area ), wid, FALSE, FALSE, 0);
  gtk_signal_connect( GTK_OBJECT(wid), "clicked",
		      GTK_SIGNAL_FUNC( alphakara_gtkprefs_ok), this);
  gtk_widget_show(wid);

  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(_main)->vbox), notebook);
  gtk_widget_show(notebook);


  gtk_signal_connect( GTK_OBJECT(_main), "delete_event",
		      GTK_SIGNAL_FUNC( alphakara_gtkprefs_delete_event ), this);

}

AlphaKaraoke::AlphaKaraokeGtkPrefs::~AlphaKaraokeGtkPrefs()
{
  for (int i=0; i<4; i++)
    {
      delete _colorDialogs[i];
    }
  delete[] _colorDialogs;
  gtk_widget_destroy(_main);
}

void AlphaKaraoke::AlphaKaraokeGtkPrefs::show()
{
  gtk_widget_show(_main);
}

void AlphaKaraoke::AlphaKaraokeGtkPrefs::hide()
{
  gtk_widget_hide(_main);
}

void AlphaKaraoke::AlphaKaraokeGtkPrefs::apply()
{
  _mother->setFont(_fontDialog->getName(), true);

  int r,g,b;
  _colorDialogs[0]->getColors(r,g,b);
  _mother->setBackgroundColor(r,g,b);  
  _colorDialogs[1]->getColors(r,g,b);
  _mother->setReadForegroundColor(r,g,b);
  _colorDialogs[2]->getColors(r,g,b);
  _mother->setUnreadForegroundColor(r,g,b);
  _colorDialogs[3]->getColors(r,g,b);
  _mother->setCurrentForegroundColor(r,g,b);

  int smooth = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_scrollSmooth))?AlphaKaraoke::SCROLL_SMOOTH:AlphaKaraoke::SCROLL_LINE;
  int type;
  if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_scrollLineFrame)))
    type = AlphaKaraoke::SCROLL_FRAMED;
  else if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_scrollLineCenter)))
    type = AlphaKaraoke::SCROLL_CENTER;
  else if ( gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(_scrollLineTop)))
    type = AlphaKaraoke::SCROLL_TOP;

  _mother->setScrollType(smooth,type);

  _mother->updatePixmapBuffer();
}

void AlphaKaraoke::AlphaKaraokeGtkPrefs::changed()
{
  //  gnome_property_box_changed( getPropertyBox());
}

void AlphaKaraoke::AlphaKaraokeGtkPrefs::showFontDialog()
{
  _fontDialog->show();
}

#endif
