/*
  Midnight, a C++ midi library to load and play midi files.
  Copyright (C) 1999 Rodrigo Reyes, reyes@chez.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "ColorDialogPrefs.h"

static gint colordialogprefs_ok(GtkWidget *widget, gpointer data)
{
  ((ColorDialogPrefs*)data)->ok();
  return TRUE;
}

static gint colordialogprefs_cancel(GtkWidget *widget, gpointer data)
{
  ((ColorDialogPrefs*)data)->cancel();
  return TRUE;
}

static gint colordialogprefs_delete_event(GtkWidget* widget, GdkEventConfigure* event, gpointer data)
{
  ((ColorDialogPrefs*)data)->cancel();
  return TRUE;
}

ColorDialogPrefs::ColorDialogPrefs(const char *title, ColorDPUnit color) : Color(color)
{
  colMain = gtk_color_selection_dialog_new(title);
  //  gtk_font_selection_dialog_set_font_name(GTK_FONT_SELECTION_DIALOG(colMain), name.c_str());
  gtk_color_selection_set_color( GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(colMain)->colorsel), Color.getArray());
  gtk_signal_connect( GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(colMain)->ok_button), "clicked",
		      GTK_SIGNAL_FUNC( colordialogprefs_ok ), this);
  gtk_signal_connect( GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(colMain)->cancel_button), "clicked",
		      GTK_SIGNAL_FUNC( colordialogprefs_cancel ), this);
  gtk_signal_connect( GTK_OBJECT(colMain), "delete_event",
		      GTK_SIGNAL_FUNC( colordialogprefs_delete_event ), this);
  Visible = false;
}

ColorDialogPrefs::~ColorDialogPrefs()
{
  gtk_widget_destroy(colMain);
}

void ColorDialogPrefs::show()
{
  if (Visible == false)
    {
      gtk_color_selection_set_color( GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(colMain)->colorsel), Color.getArray());
      gtk_widget_show(colMain);
      Visible = true;
    }
}
void ColorDialogPrefs::hide()
{
  gtk_widget_hide(colMain);
  Visible = false;
}

void ColorDialogPrefs::ok()
{
  //  fontName = gtk_font_selection_dialog_get_font_name(GTK_FONT_SELECTION_DIALOG(colMain));
  gtk_color_selection_get_color( GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(colMain)->colorsel), Color.getArray());
  hide();
}

void ColorDialogPrefs::cancel()
{
  hide();
}
