/*
  Midnight, a C++ midi library to load and play midi files.
  Copyright (C) 1999 Rodrigo Reyes, reyes@chez.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "KaraokeBase.h"
#include <iostream.h>
#include <sys/stat.h>
#include <stdio.h>
#include <unistd.h>
#include "CharsetConv.h"

KaraokeBase::KaraokeBase(PlayerInterface *player) : _guiplayer(player)
{
}


ostream &operator << (ostream &o, vector<pair<unsigned long, string> > &vec)
{
  o << "LYRICS[" << endl;
  for (vector<pair<unsigned long, string> >::iterator cur = vec.begin(); cur != vec.end(); cur++)
    {
      o << cur->second << endl;
    }
  o << "]LYRICS" << endl;
  return o;
}

vector<pair<unsigned long, string> >::iterator & KaraokeBase::findNextEvent(unsigned long tick)
{
  if (_lyriter == _text.end())
    {
      _lyriter = _text.begin();
    }

  if (_lyriter->first <= tick)
    {
      // search forward

      if (_lyriter == _text.end())
      	return _lyriter;

      ++_lyriter;

      bool stop = false;
      while ((_lyriter != _text.end()) && !stop)
	{
	  if (_lyriter->first < tick)
	    ++_lyriter;
	  else stop = true;
	}

      if (_lyriter->first > tick)
	--_lyriter;

    }
  else if (_lyriter->first > tick)
    {
      // search backward
	  
      if (_lyriter == _text.begin())
	return _lyriter;

      bool stop = false;
      while ((_lyriter != _text.begin()) && !stop)
	{
	  if (_lyriter->first > tick)
	    --_lyriter;
	  else stop = true;
	}
    }
  return _lyriter;
}

vector<pair<unsigned long, string> >::iterator & KaraokeBase::getEventIterator()
{
  return _lyriter;
}

vector<pair<unsigned long, string> >::iterator KaraokeBase::getEventBegin()
{
  return _text.begin();
}

vector<pair<unsigned long, string> >::iterator KaraokeBase::getEventEnd()
{
  return _text.end();
}

void KaraokeBase::setGUIPlayer(PlayerInterface *mdngp)
{
  _guiplayer = mdngp;
}

void KaraokeBase::clearLyrics()
{
  _text.clear();
  _lyriter = _text.end();
}

void KaraokeBase::addLyricUnit(pair<unsigned long, string> lunit)
{
  _text.push_back(lunit);
}

void KaraokeBase::initLyrics()
{
  _lyriter = _text.begin();
  CharsetConv::convert(_text);
  
}

PlayerInterface* KaraokeBase::getGUIPlayer()
{
  return _guiplayer;
}
