/*
  Midnight, a C++ midi library to load and play midi files.
  Copyright (C) 1999 Rodrigo Reyes, reyes@chez.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "MidnightDevicePrefs.h"


static void mdp_ok(GtkWidget* w, gpointer data)
{
  ((MidnightDevicePrefs*)data)->applyChanges();
   delete ((MidnightDevicePrefs*)data);
}

static void mdp_cancel(GtkWidget* w, gpointer data)
{
  ((MidnightDevicePrefs*)data)->cancel();
  delete ((MidnightDevicePrefs*)data);
}

static gint mdp_delete_event(GtkWidget* widget, GdkEventConfigure* event, gpointer data)
{
  ((MidnightDevicePrefs*)data)->cancel();
  delete ((MidnightDevicePrefs*)data);
  return TRUE;
}

MidnightDevicePrefs::MidnightDevicePrefs(MidnightConfiguration *config) : _config(config)
{
  _deviceName = NULL;

  _mainWindow = gtk_dialog_new();
  gtk_widget_show(_mainWindow);

  GtkWidget *actionarea = GTK_DIALOG(_mainWindow)->action_area;
  GtkWidget *vbox = GTK_DIALOG(_mainWindow)->vbox; 

  gtk_signal_connect( GTK_OBJECT(_mainWindow), "delete_event",
		      GTK_SIGNAL_FUNC( mdp_delete_event ), this);


  {
	
  }

  {
    char *titles[] = { "Description", "Type" };
	
    _list = gtk_clist_new_with_titles(2, titles);
    gtk_clist_set_selection_mode( GTK_CLIST(_list), GTK_SELECTION_SINGLE);

    populateList();
	
    gtk_widget_show(_list);
    gtk_box_pack_start( GTK_BOX( vbox ), _list, TRUE, TRUE, 0 );
  }

  {
    GtkWidget *hbox = gtk_hbox_new(TRUE, 10);
    gtk_widget_show(hbox);

    {
      GtkWidget *button = gtk_button_new_with_label("Ok");
      gtk_widget_show(button);
      gtk_box_pack_start( GTK_BOX( hbox ), button, FALSE, FALSE, 0 );
      gtk_signal_connect( GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(mdp_ok), this);
    }

    {
      GtkWidget *button = gtk_button_new_with_label("Cancel");
      gtk_widget_show(button);
      gtk_box_pack_start( GTK_BOX( hbox ), button, FALSE, FALSE, 0 );
      gtk_signal_connect( GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(mdp_cancel), this);
    }

    gtk_box_pack_start( GTK_BOX( actionarea ), hbox, FALSE, FALSE, 0 );
  }

  gtk_widget_show(vbox);
  gtk_widget_show(_mainWindow);

}


MidnightDevicePrefs::~MidnightDevicePrefs()
{
  gtk_widget_destroy(_mainWindow);
}

void MidnightDevicePrefs::populateList()
{
  const char *row[2];
  static char *string_external = "External";
  static char *string_internal = "Internal";

  gtk_clist_clear( GTK_CLIST(_list) );

  int elemIdx;
  _elems = _config->getElements(_deviceName, elemIdx);
  for (vector<MidnightConfElement>::iterator i=_elems.begin(); i!=_elems.end(); i++)
    {
      row[0] = i->getDescription();
      if (i->getOutput() == MidnightConfiguration::SYNTH_OUTPUT)
	row[1] = string_internal;
      else row[1] = string_external;

      gtk_clist_append(GTK_CLIST(_list), (char**)row);
    }

  for (int i=0; i<2; i++)
    {
      gtk_clist_set_column_auto_resize ( GTK_CLIST(_list), i, TRUE);
    }

  if (elemIdx>=0)
    {
      gtk_clist_select_row( GTK_CLIST(_list), elemIdx, 0);
      gtk_clist_select_row( GTK_CLIST(_list), elemIdx, 1);
    }

}

void MidnightDevicePrefs::applyChanges()
{
  GList *list = GTK_CLIST( _list )->selection;
  while (list != NULL)
    {
      int row = GPOINTER_TO_INT(list->data);

      MidnightConfElement & mcf = _elems[row];

      _config->configureWith(mcf);
      list = list->next;
    }
}

void MidnightDevicePrefs::cancel()
{
  
}
