/*
  Midnight, a C++ midi library to load and play midi files.
  Copyright (C) 1999 Rodrigo Reyes, reyes@chez.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "AlphaKaraoke.h"
#include "MidnightGUIPlayer.h"

void mguip_select_device(GtkWidget* w, gpointer data)
{
  ((MidnightGUIPlayer*)data)->selectDevice();
}

void mguip_open(GtkWidget* w, gpointer data)
{
  ((MidnightGUIPlayer*)data)->openSelector();
}

void mguip_save_config(GtkWidget* w, gpointer data)
{
  ((MidnightGUIPlayer*)data)->saveConfig();
}


void mguip_quit(GtkWidget* w, gpointer data)
{
  gtk_main_quit();
}

void mguip_show_hide_panel(GtkWidget* w, gpointer data)
{
  ((MidnightGUIPlayer*)data)->toggleShowKaraokePanel();
}

void mguip_karaoke_prefs(GtkWidget* w, gpointer data)
{
  ((MidnightGUIPlayer*)data)->showKaraokePrefs();
}

void mguip_show_hide_list(GtkWidget* w, gpointer data)
{
  ((MidnightGUIPlayer*)data)->toggleShowListPanel();
}

int mguip_data_waiting(gpointer data)
{
  ((MidnightGUIPlayer*)data)->consumeData();
  return 1;
}

MidnightGUIPlayer::MidnightGUIPlayer(MidnightPlayerManager* manager) : _playermanager(manager)
{
}
void MidnightGUIPlayer::init()
{
  GUIPlayer::init();

  _selectDevice = NULL;
  _karaokeHidden = true;

  // attach the menubar
  gtk_window_set_policy( GTK_WINDOW(getWidget()), TRUE, TRUE, TRUE);

  gtk_signal_connect( GTK_OBJECT(getWidget()),
		      "destroy", GTK_SIGNAL_FUNC(mguip_quit), this);

  gtk_timeout_add(50, mguip_data_waiting, this);

  _karaoke = new AlphaKaraoke(this);

  _titleList = new MidnightList(this);
}

void MidnightGUIPlayer::attachMenuToMain(GtkWidget* main, GtkWidget* vbox)
{
  _internalMenuBarLocation = gtk_vbox_new(FALSE,0);

  GtkAccelGroup *accel_group = gtk_accel_group_new();
  _itemFactory = gtk_item_factory_new( GTK_TYPE_MENU_BAR, "<main>", accel_group);
     
  gtk_accel_group_attach( accel_group, GTK_OBJECT( getWidget() ));
      
  itemFactoryAdd(_itemFactory);

  GtkWidget *menubar = gtk_item_factory_get_widget( _itemFactory, "<main>");
  gtk_container_add( GTK_CONTAINER(_internalMenuBarLocation), menubar);
  gtk_widget_show( menubar );
  gtk_widget_show(_internalMenuBarLocation);

  gtk_box_pack_start( GTK_BOX(vbox), _internalMenuBarLocation, FALSE, TRUE, 0);
}

 
MidnightGUIPlayer::~MidnightGUIPlayer()
{
}


void MidnightGUIPlayer::itemFactoryAdd(GtkItemFactory *item_factory)
{
  static GtkItemFactoryEntry menu_items[] = {
    {"/File", NULL, NULL, 0, "<Branch>" },
    {"/File/_Open...", "<control>O", GTK_SIGNAL_FUNC(mguip_open), (unsigned int)this, NULL},
    {"/File/sep1", NULL, NULL, 0, "<Separator>"},
    {"/File/Show list panel", "<control>L", GTK_SIGNAL_FUNC(mguip_show_hide_list), (unsigned int)this, "<CheckItem>"},
    {"/File/sep2", NULL, NULL, 0, "<Separator>"},
    {"/File/_Quit", "<control>Q", GTK_SIGNAL_FUNC(mguip_quit), (unsigned int)this, NULL},

    {"/Karaoke", NULL, NULL, 0, "<Branch>" },
    {"/Karaoke/Show panel", "<control>P", GTK_SIGNAL_FUNC(mguip_show_hide_panel), (unsigned int)this, "<ToggleItem>"},
    {"/Karaoke/Preferences...", NULL, GTK_SIGNAL_FUNC(mguip_karaoke_prefs), (unsigned int)this, NULL},

    {"/Configuration", NULL, NULL, 0, "<Branch>" },
    {"/Configuration/_Midi device...", "<control>M", GTK_SIGNAL_FUNC(mguip_select_device), (unsigned int)this, NULL},
    {"/Configuration/sep1", NULL, NULL, 0, "<Separator>"},
    {"/Configuration/Save _Configuration", "<control>C", GTK_SIGNAL_FUNC(mguip_save_config), (unsigned int)this, NULL},
  };

  gint nbmenus = sizeof(menu_items) / sizeof( menu_items[0]);
      
  gtk_item_factory_create_items( item_factory, nbmenus, menu_items, NULL);

  _showListPanelItem = gtk_item_factory_get_widget(item_factory, "/File/Show list panel" );
  _showKaraokePanelItem = gtk_item_factory_get_widget(item_factory, "/Karaoke/Show panel");
}
  

void MidnightGUIPlayer::selectDevice()
{
  MidnightConfiguration *config = _playermanager->lockConfiguration();
  if (_selectDevice == NULL)
    _selectDevice = new SelectDevicePrefs(config, this);
  _playermanager->unlockConfiguration();
}

void MidnightGUIPlayer::newFileSelected()
{
  stop();
  checkNotPlaying();

  int startrow = _titleList->addTitle(string(getFileName()));
  _titleList->select(startrow);
  _titleList->playSelection();

  //  _playermanager->setSelectedFile(getFileName());
  //  setSongInfo(getFileName());
  //  _playermanager->play();
  play();
}

void MidnightGUIPlayer::stop()
{
  GUIPlayer::stop();
  _playermanager->stop();
  _playermanager->setPosition(0);
}

void MidnightGUIPlayer::play()
{
  //  _playermanager->play();
  
  if (_playermanager != NULL)
    {
      if (_playermanager->isPlaying())
	{
	  setPlayPixAsPause();
	  _playermanager->pause();
	}
      else
	{
	  setPlayPixAsPlay();
	  _playermanager->play();
	}
    }
}



void MidnightGUIPlayer::previousSong()
{
  _titleList->previous();
}

void MidnightGUIPlayer::nextSong()
{
  _titleList->next();
}

void MidnightGUIPlayer::selectSong(string& filename)
{
  stop();
  checkNotPlaying();
  setFileName(filename.c_str());
  _playermanager->setSelectedFile(filename.c_str());
  //  setSongInfo(filename.c_str());
  play();
}

void MidnightGUIPlayer::closedSelectDeviceWindow()
{
  _selectDevice = NULL;
}

void MidnightGUIPlayer::saveConfig()
{
  _playermanager->saveConfig();
}

void MidnightGUIPlayer::setKaraokePanelVisibility(bool v)
{
  GtkWidget *wid =  gtk_item_factory_get_widget( _itemFactory, "/Karaoke/Show karaoke panel");
  wid = 0;
}

void MidnightGUIPlayer::toggleShowKaraokePanel()
{
  if (_karaoke != NULL)
    {
      if (_karaoke->isShowing())
	{
	  _karaoke->hide();
	}
      else
	{
	  _karaoke->show();
	}
    }
}

void MidnightGUIPlayer::toggleShowListPanel()
{
  if (_titleList != NULL)
    {
      if (_titleList->isShowing())
	{
	  _titleList->hide();
	}
      else
	{
	  _titleList->show();
	}
    }
}


void MidnightGUIPlayer::setSongAt(unsigned long ticks)
{
  _playermanager->setPosition(ticks);
  if (_karaoke != NULL)
    {
      _karaoke->update(ticks);
    }
}

void MidnightGUIPlayer::checkNotPlaying()
{
  _playermanager->pause();
  while (_playermanager->checkNotPlaying() == false)
    {
      usleep(1000000);
    }
}

void MidnightGUIPlayer::consumeData()
{
  int value1;
  string buffer;

  ///     if (_playermanager->isPlaying() == false)
  ///	return;

  while (true)
    {
      switch( _playermanager->pollEvent(value1, buffer))
	{
	case MidnightPlayerManager::TICK_MS:
	  setPosition(value1, buffer.c_str(), false);
	  break;
	case MidnightPlayerManager::TICK_LYRICS:
	  if (_karaoke)
	    _karaoke->ticked(value1);
	  break;

	case MidnightPlayerManager::END_LYRICS_TEXT:
	  if (_karaoke)
	    _karaoke->initLyrics();
	  
	  break;

	case MidnightPlayerManager::CLEAR_LYRICS_TEXT:
	  if (_karaoke)
	    _karaoke->clearLyrics();
	  break;
	case MidnightPlayerManager::ADD_LYRICS_TEXT:
	  {
	    if (_karaoke)
	      {
		_karaoke->addLyricUnit(make_pair(value1, buffer.c_str()));
		//		cerr << "kara : " << buffer << endl;
	      }
	  }
	  break;
	case MidnightPlayerManager::SET_FILE_LENGTH:
	  adjustTimeScale(value1);
	  break;

	case MidnightPlayerManager::SONG_ENDED:
	  //	      stop();
	  //	  cerr << "SONG ENDED RECEIVED" << endl;
	  {
	    GUIPlayer::stop();
	    
	    _playermanager->setPosition(0);
	    
	    if (_karaoke)
	      {
		_karaoke->initLyrics();
	      }
	    if (_titleList)
	      {
		_titleList->next();
	      }
	  }
	  break;

	case MidnightPlayerManager::CLEAR_TITLE_TEXT:
	  _titleText.clear();
	  break;
	case MidnightPlayerManager::ADD_TITLE_TEXT:
	  _titleText.push_back(buffer.c_str());
	  break;
	case MidnightPlayerManager::END_TITLE_TEXT:
	  break;
	  
	case MidnightPlayerManager::CLEAR_INFO_TEXT:
	  _infoText.clear();
	  break;
	case MidnightPlayerManager::ADD_INFO_TEXT:
	  _infoText.push_back(buffer.c_str());
	  break;
	case MidnightPlayerManager::END_INFO_TEXT:
	  break;

	case MidnightPlayerManager::UPDATE_INFO:
	  {
	    string inf;
	    for (unsigned int i=0; i<_titleText.size(); i++)
	      {
		inf += _titleText[i] + "\n";
	      }

	    for (unsigned int i=0; i<_infoText.size(); i++)
	      {
		inf += _infoText[i] + "\n";
	      }
	    if (inf.length() == 0)
	      {
		setSongInfo((getFileName()==0)?"No information available":getFileName());
	      }
	    else
	      setSongInfo(inf.c_str());

	    if (_karaoke != 0)
	      {
		string val;

		for (unsigned int i=0; i<_titleText.size(); i++)
		  {
		    if (i>0)
		      val += " - ";
		    val += _titleText[i];
		  }

		if (_titleText.size()>0)
		  _karaoke->setStatus(val);
		else
		  _karaoke->setStatus("Unknown song");
	      }



	  }
	  break;

	default:
	  return;
	}
    }
}

void MidnightGUIPlayer::addToList(vector<string> &lst)
{
  if (_titleList != 0)
    {
      int startrow = _titleList->addTitles(lst);
      _titleList->select(startrow);
      _titleList->playSelection();
    }
}

void MidnightGUIPlayer::toggleMenuItem_show_karaoke(bool val)
{
  //  gtk_check_menu_item_set_active      (GtkCheckMenuItem *check_menu_item,
  //  				       gboolean	  is_active);
  //  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(_showKaraokePanelItem), val);
  GTK_CHECK_MENU_ITEM(_showKaraokePanelItem)->active = (val?1:0);
}

void MidnightGUIPlayer::toggleMenuItem_show_list(bool val)
{
  //  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(_showListPanelItem), val);
  GTK_CHECK_MENU_ITEM(_showListPanelItem)->active = (val?1:0);
}

void MidnightGUIPlayer::showKaraokePrefs()
{
  if (_karaoke != 0)
    {
      _karaoke->showPreferences();
    }
}


void MidnightGUIPlayer::setFeature(string key, string value)
{
  MidnightConfiguration *config = _playermanager->lockConfiguration();
  config->setFeature(key, value);
  _playermanager->unlockConfiguration();
}

string MidnightGUIPlayer::getFeature(string key, string def)
{
  MidnightConfiguration *config = _playermanager->lockConfiguration();
  string res = config->getFeature(key, def);
  _playermanager->unlockConfiguration();
  return res;
}

void MidnightGUIPlayer::setIntFeature(string key, int value)
{
  MidnightConfiguration *config = _playermanager->lockConfiguration();
  config->setIntFeature(key, value);
  _playermanager->unlockConfiguration();
}

int MidnightGUIPlayer::getIntFeature(string key, int def)
{
  MidnightConfiguration *config = _playermanager->lockConfiguration();
  int res = config->getIntFeature(key, def);
  _playermanager->unlockConfiguration();
  return res;
}

void MidnightGUIPlayer::flushConfiguration()
{
  MidnightConfiguration *config = _playermanager->lockConfiguration();
  config->save();
  _playermanager->unlockConfiguration();
}
