/*
  Midnight, a C++ midi library to load and play midi files.
  Copyright (C) 1999 Rodrigo Reyes, reyes@chez.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "MidnightList.h"

static void mdnlist_delete_entry(GtkWidget* w, gpointer data)
{
  ((MidnightList*)data)->deleteEntry();
}

static void mdnlist_delete_entry_all(GtkWidget* w, gpointer data)
{
  ((MidnightList*)data)->deleteAll();
  //  ((MidnightDevicePrefs*)data)->applyChanges();
  //   delete ((MidnightDevicePrefs*)data);
}

static void mdnlist_play_sel(GtkWidget* w, gpointer data)
{
  ((MidnightList*)data)->playSelection();
}

static gint mdnlist_delete_event(GtkWidget* widget, GdkEventConfigure* event, gpointer data)
{
  ((MidnightList*)data)->hide();
  return TRUE;
}

static void mdnlist_row_selected(GtkWidget*widget, gint row, gint column, GdkEventButton*event, gpointer data)
{
  if (event == 0)
    return;

  if (event->type == GDK_2BUTTON_PRESS)
    {
      ((MidnightList*)data)->playSelection();
    }

}

void mdnlist_data_dropped  (GtkWidget          *widget,
			      GdkDragContext     *context,
			      gint                x,
			      gint                y,
			      GtkSelectionData   *data,
			      guint               info,
			      guint               time,
			      gpointer            mdnlst)
  {
  if ((data->length >= 0) && (data->format == 8))
    {
      //      g_print ("Received \"%s\" in label\n", (gchar *)data->data);

      //      ((GUIPlayer*)gplayer)->droppedData((gchar *)data->data);

      StringTokenizer st((gchar *)data->data,"\n\r\t");
      vector<string> lst;

      while (st.hasNext())
	{
	  string cur = st.next();
	  
	  if (cur.find("file://")==0)
	    {
	      lst.push_back(cur.substr(7, cur.size()-7));
	    }
	  else if (cur.find("file:")==0)
	    {
	      lst.push_back(cur.substr(5, cur.size()-5));
	    }
	  else 
	    {
	      lst.push_back(cur);
	    }
	  
	}

      ((MidnightList*)mdnlst)->addTitles(lst);

      gtk_drag_finish (context, TRUE, FALSE, time);
      return;
    }
  
  gtk_drag_finish (context, FALSE, FALSE, time);
}

enum {
  TARGET_STRING,
  TARGET_ROOTWIN
};

static GtkTargetEntry target_table[] = {
  { "STRING",     0, TARGET_STRING },
  { "text/plain", 0, TARGET_STRING },
  { "application/x-rootwin-drop", 0, TARGET_ROOTWIN }
};

MidnightList::MidnightList(PlayerInterface *player) : _player(player)
{
  _mainWindow = gtk_window_new( GTK_WINDOW_TOPLEVEL );
  _isShowing = false;

  GtkWidget *vbox = gtk_vbox_new(FALSE, 0);

  gtk_drag_dest_set (_mainWindow,
		     GTK_DEST_DEFAULT_ALL,
		     target_table, sizeof(target_table) / sizeof(target_table[0])-1, /* no rootwin */
		     (GdkDragAction)(GDK_ACTION_COPY | GDK_ACTION_MOVE));
  
  gtk_signal_connect( GTK_OBJECT(_mainWindow), "drag_data_received",
		      GTK_SIGNAL_FUNC( mdnlist_data_dropped ), this);

  gtk_signal_connect( GTK_OBJECT(_mainWindow), "delete_event",
		      GTK_SIGNAL_FUNC( mdnlist_delete_event ), this);

  {
    GtkWidget* scroller = gtk_scrolled_window_new(NULL,NULL);
    gtk_widget_show(scroller);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroller), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

    char *titles[] = { "Title", "Type" };
	
    _list = gtk_clist_new_with_titles(1, titles);
    gtk_clist_set_selection_mode( GTK_CLIST(_list), GTK_SELECTION_SINGLE);

    gtk_signal_connect( GTK_OBJECT(_list), "select_row",
			GTK_SIGNAL_FUNC( mdnlist_row_selected ), this);

    gtk_widget_show(_list);
    
    //    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scroller), _list);
    gtk_container_add(GTK_CONTAINER(scroller), _list);

    gtk_box_pack_start( GTK_BOX( vbox ), scroller, TRUE, TRUE, 0 );
  }

  {
    GtkWidget *hbox = gtk_hbox_new(true, 5);
    gtk_widget_show(hbox);
    {
      GtkWidget *button = gtk_button_new_with_label("Delete");
      gtk_widget_show(button);
      gtk_box_pack_start( GTK_BOX( hbox ), button, FALSE, TRUE, 0 );
      gtk_signal_connect( GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(mdnlist_delete_entry), this);
    }
    {
      GtkWidget *button = gtk_button_new_with_label("Delete all");
      gtk_widget_show(button);
      gtk_box_pack_start( GTK_BOX( hbox ), button, FALSE, TRUE, 0 );
      gtk_signal_connect( GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(mdnlist_delete_entry_all), this);
    }
    {
      GtkWidget *button = gtk_button_new_with_label("Play the selection");
      gtk_widget_show(button);
      gtk_box_pack_start( GTK_BOX( hbox ), button, FALSE, TRUE, 0 );
      gtk_signal_connect( GTK_OBJECT(button), "clicked", GTK_SIGNAL_FUNC(mdnlist_play_sel), this);
    }

    gtk_box_pack_start( GTK_BOX( vbox ), hbox, FALSE, FALSE, 0 );
  }
  
  gtk_widget_show(vbox);
  gtk_container_add( GTK_CONTAINER(_mainWindow), vbox);
}


int MidnightList::addTitle(string title)
{
  _titles.push_back(title);

  const char *row[1];
  row[0] = title.c_str();
  return gtk_clist_append(GTK_CLIST(_list), (char**)row);
}

int MidnightList::addTitles(vector<string>& titles)
{
  int val = -1;
  for (unsigned int i=0; i<titles.size(); i++)
    {
      int c = addTitle(titles[i]);
      if (val == -1)
	val = c;
    }
  return val;
}

void MidnightList::playSelection()
{
  GList *list = GTK_CLIST( _list )->selection;
  if (list != NULL)
    {
      int row = GPOINTER_TO_INT(list->data);

      ((GUIPlayer*)_player)->selectSong(_titles[row]);
    }
}

void MidnightList::select(int row)
{
  if ((row>=0) && (row < (int)_titles.size()))
    {
      gtk_clist_select_row( GTK_CLIST(_list), row, 0);
      if (gtk_clist_row_is_visible( GTK_CLIST(_list), row) == FALSE)
	gtk_clist_moveto( GTK_CLIST(_list), row, 0,1.0,0.0);
    }      
}

void MidnightList::next()
{
  int row = -1;
  GList *list = GTK_CLIST( _list )->selection;
  if (list != NULL)
    {
      row = GPOINTER_TO_INT(list->data);
    } 

  row++;

  if (row < (int)_titles.size())
    {
      gtk_clist_select_row( GTK_CLIST(_list), row, 0);
      _player->selectSong(_titles[row]);
    }
}

void MidnightList::previous()
{
  int row = 1;
  GList *list = GTK_CLIST( _list )->selection;
  if (list != NULL)
    {
      row = GPOINTER_TO_INT(list->data);
    } 

  row--;

  if ((row>=0) && (row < (int)_titles.size()))
    {
      gtk_clist_select_row( GTK_CLIST(_list), row, 0);
      _player->selectSong(_titles[row]);
    }
}

void MidnightList::deleteAll()
{
  gtk_clist_clear( GTK_CLIST(_list) );
}

void MidnightList::deleteEntry()
{
  GList *list = GTK_CLIST( _list )->selection;
  if (list != NULL)
    {
      int row = GPOINTER_TO_INT(list->data);

      vector<string>::iterator it = _titles.begin();
      it+= row;

      gtk_clist_remove( GTK_CLIST(_list), row);
      _titles.erase(it);
      list = list->next;

      if ((GTK_CLIST(_list)->rows)> row)
	gtk_clist_select_row( GTK_CLIST(_list), row, 0);
      else if ((GTK_CLIST(_list)->rows)>0)
	gtk_clist_select_row( GTK_CLIST(_list), row-1, 0);
    }
}

bool MidnightList::isShowing()
{
  return _isShowing;
}

void MidnightList::show()
{
  if (_isShowing == false)
    {
      _isShowing = true;
      gtk_widget_show(_mainWindow);
    }
}

void MidnightList::hide()
{
  if (_isShowing == true)
    {
      _isShowing = false;
      gtk_widget_hide(_mainWindow);
    }

  _player->toggleMenuItem_show_list(false);
}
