/*
  Midnight, a C++ midi library to load and play midi files.
  Copyright (C) 1999 Rodrigo Reyes, reyes@chez.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "MidnightPlayerManager.h"


void *midnmanager_run(void *data)
{
  while (true)
    {
      ((MidnightPlayerManager*)data)->internalRun();
      usleep(500000);
    }
}

MidnightPlayerManager::MidnightPlayerManager(MidnightConfiguration*config)  : _config(config)
{
  pthread_t runner;
  pthread_create (&runner, NULL, midnmanager_run, this);

  _player = NULL;
  _file = NULL;
  _device = NULL;

  _configLock = false;

  _config->addMidnightConfigurationListener(this);
}

void MidnightPlayerManager::load(string filename)
{
  if (_player != NULL)
    {
      ifstream in(filename.c_str());

      if (!in)
	return;

      // stop();
      _player->stop();
      delete _file;
      _file = new MidiFile(in);
      if (!_file)
	return;
      
      _player->setFile(_file);
      //	  adjustTimeScale(_player->totalTicks());
      //      _player->stop();
      sendFileInformation();
      //      _player->play();
      return;
      //	  delete _player;
      //return;
    }

  if (_device == NULL)
    {
      _device = _config->lockDevice();
    }

  ifstream in(filename.c_str());
  if (!in)
    {
      return;
    }
      

  _file = new MidiFile(in);
  if (!_file)
    return;
  _player = new LocalMidiPlayer(this, _file, _device);
  sendFileInformation();
      
  //  _player->play();

}


void MidnightPlayerManager::sendFileInformation()
{
  if ((_file==NULL)||(_device==NULL)||(_player==NULL))
    return;
      
  // 
  // send total track length
  //

  _controllerqueue.push(InfoBag(CONTROLLER_TOTAL_LENGTH, _player->totalTicks()));

  //
  // send lyrics
  //
  vector<pair<unsigned long, MidiEvent*> >  lyrics;
  vector<string> titles,info;
  string language;
  //  MidiUtils::extractLyrics(_player->getEvents(),  lyrics, language, titles, info);
  MidiUtils::extractLyrics(_file,  lyrics, language, titles, info);

      
  //
  // clear the lyrics...
  _controllerqueue.push(InfoBag(CONTROLLER_CLEAR_LYRICS, 0));
      
  //
  // ...And send the new ones
  for (vector<pair<unsigned long, MidiEvent*> >::iterator i = lyrics.begin(); i!= lyrics.end(); i++)
    {
      const char *text = ((MidiMetaTextEvent*)i->second->getData())->getText();
      _controllerqueue.push(InfoBag(CONTROLLER_ADD_LYRICS, i->first, text));
    }

  _controllerqueue.push(InfoBag(CONTROLLER_ADD_LYRICS_END, 0));

  //
  // clear the titles info...
  _controllerqueue.push(InfoBag(CONTROLLER_CLEAR_TITLE, 0));
  // ...And send the new ones
  for (vector<string>::iterator i = titles.begin(); i!= titles.end(); i++)
    {
      _controllerqueue.push(InfoBag(CONTROLLER_ADD_TITLE, 0, *i));
    }
  _controllerqueue.push(InfoBag(CONTROLLER_ADD_TITLE_END, 0));

  //
  // clear the info...
  _controllerqueue.push(InfoBag(CONTROLLER_CLEAR_INFO, 0));
  // ...And send the new ones
  for (vector<string>::iterator i = info.begin(); i!= info.end(); i++)
    {
      _controllerqueue.push(InfoBag(CONTROLLER_ADD_INFO, 0, *i));
    }
  _controllerqueue.push(InfoBag(CONTROLLER_ADD_INFO_END, 0));

  _controllerqueue.push(InfoBag(CONTROLLER_UPDATE_INFO, 0));

}

ostream &operator << (ostream &o, InfoBag&bag)
{
  o << "[" << bag.Command << "," << bag.Value << "]";
  return o;
}
