/*
  Midnight, a C++ midi library to load and play midi files.
  Copyright (C) 1999 Rodrigo Reyes, reyes@chez.com

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef _THREADED_QUEUE_H
#define _THREADED_QUEUE_H

#include <pthread.h>
#include <queue>

template <class T> 
class ThreadedQueue
{
 private:
  
  queue<T> _queue;
  pthread_mutex_t _mutex;

 public:
  ThreadedQueue()
    {
      pthread_mutex_init (&_mutex, NULL);
      
    }

  void push(T object)
    {
      pthread_mutex_lock (&_mutex);
      
      _queue.push(object);

      pthread_mutex_unlock (&_mutex);
    }

  T pop()
    {
      pthread_mutex_lock (&_mutex);

      T ob = _queue.front();
      _queue.pop();
      pthread_mutex_unlock (&_mutex);

      return ob;
    }

  T& flavor()
    {
      pthread_mutex_lock (&_mutex);
      T &ob = _queue.front();
      pthread_mutex_unlock (&_mutex);
      return ob;
    }
  
  bool empty()
    {
      pthread_mutex_lock (&_mutex);

      bool result = _queue.empty();
      pthread_mutex_unlock (&_mutex);

      return result;
    }

};

#endif
